@@ -19,6 +19,7 @@ package hashivault
 import (
 	"context"
 	"crypto"
+	"crypto/ed25519"
 	"encoding/base64"
 	"encoding/json"
 	"errors"
@@ -220,17 +221,30 @@ func (h *hashivaultClient) fetchPublicKey(_ context.Context) (crypto.PublicKey,
 		return nil, fmt.Errorf("could not parse transit key keys data as map[string]interface{}")
 	}
 
-	publicKeyPem, ok := keyMap["public_key"]
+	publicKey, ok := keyMap["public_key"]
 	if !ok {
 		return nil, errors.New("failed to read transit key keys: corrupted response")
 	}
 
-	strPublicKeyPem, ok := publicKeyPem.(string)
+	strPublicKey, ok := publicKey.(string)
 	if !ok {
 		return nil, fmt.Errorf("could not parse public key pem as string")
 	}
+	// vault returns the key type in the "name" field
+	if keyType := keyMap["name"]; keyType == "ed25519" {
+		// vault returns ed25519 public keys as base64 encoding of
+		// the raw bytes of the key
+		decodedPublicKey, err := base64.StdEncoding.DecodeString(strPublicKey)
+		if err != nil {
+			return nil, fmt.Errorf("failed to base64 decode ed25519 public key: %w", err)
+		}
+		if keyLen := len(decodedPublicKey); keyLen != ed25519.PublicKeySize {
+			return nil, fmt.Errorf("decoded ed25519 public key length is %d, should be %d", keyLen, ed25519.PublicKeySize)
+		}
+		return ed25519.PublicKey(decodedPublicKey), nil
+	}
 
-	return cryptoutils.UnmarshalPEMToPublicKey([]byte(strPublicKeyPem))
+	return cryptoutils.UnmarshalPEMToPublicKey([]byte(strPublicKey))
 }
 
 func (h *hashivaultClient) public() (crypto.PublicKey, error) {
@@ -276,9 +290,10 @@ func (h hashivaultClient) sign(digest []byte, alg crypto.Hash, opts ...signature
 	}
 
 	signResult, err := client.Write(fmt.Sprintf("/%s/sign/%s%s", h.transitSecretEnginePath, h.keyPath, hashString(alg)), map[string]interface{}{
-		"input":       base64.StdEncoding.Strict().EncodeToString(digest),
-		"prehashed":   alg != crypto.Hash(0),
-		"key_version": keyVersion,
+		"input":               base64.StdEncoding.Strict().EncodeToString(digest),
+		"prehashed":           alg != crypto.Hash(0),
+		"key_version":         keyVersion,
+		"signature_algorithm": "pkcs1v15",
 	})
 	if err != nil {
 		return nil, fmt.Errorf("transit: failed to sign payload: %w", err)
